/*
 *  Pythagoras.cpp
 *  Pythagoras
 *
 *  Created by NYU User on 10/27/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include <cmath>
using namespace std;

#include <GLTools.h>			//OpenGL Toolkit
#include <GLShaderManager.h>	//Shader Manager Class
#include <GLFrustum.h>

GLFrustum frustum;
M3DMatrix44f modelView;
GLShaderManager shaderManager;

GLBatch square;
GLBatch circle;

const size_t d = 3;	//number of dimensions
GLfloat r = .5f;	//radius of circle

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
}

void SetupRC(GLint framebufferWidth, GLint framebufferHeight)	//rendering context
{
	const GLfloat aspectRatio = (GLfloat)framebufferHeight / framebufferWidth;
	frustum.SetOrthographic(
		-1.0f, 1.0f,
		-aspectRatio, aspectRatio,
		-1.0, 1.0
	);
	m3dLoadIdentity44(modelView);
	
	glClearColor(0.0f, 0.0f, 1.0f, 1.0f);	//blue background
	shaderManager.InitializeStockShaders();
	
	//Center of top edge of big square is at the origin.
	//Length of edge of big square is 2.0.
	GLfloat sv[] = {
		r, 0.0f, 0.0f,	//upper right vertex of biggest square
		-r, 0.0f, 0.0f,	//upper left
		-r, -2 * r, 0.0f,
		r, -2 * r, 0.0f
	};
	
	//number of three-dimensional vertices
	const size_t sn = sizeof sv / (d * sizeof sv[0]);
	
	square.Begin(GL_LINE_LOOP, sn);
	square.CopyVertexData3f(sv);
	square.End();
	
	const size_t cn = 50;	//number of points in semicircle
	GLfloat cv[d * cn];
	
	for (size_t i = 0; i < cn; ++i) {
		const GLfloat theta = M_PI * i / (cn - 1);
		cv[d * i + 0] = r * cosf(theta);	//x
		cv[d * i + 1] = r * sinf(theta);	//y
		cv[d * i + 2] = 0.0f;			//z
	}
	
	circle.Begin(GL_LINE_STRIP, cn);
	circle.CopyVertexData3f(cv);
	circle.End();
}

void display(GLfloat theta)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	const GLfloat vWhite[] = {1.0f, 1.0f, 1.0f, 1.0f};
	
	M3DMatrix44f modelViewProjection;
	m3dMatrixMultiply44(modelViewProjection, frustum.GetProjectionMatrix(), modelView);
	shaderManager.UseStockShader(GLT_SHADER_FLAT, modelViewProjection, vWhite);
	
	square.Draw();	//the big square on the bottom
	circle.Draw();	//the semicircle on top of it
	
	//Draw the two little squares by connecting 6 points.
	const size_t n = 6;
	
	const GLfloat rightTheta = theta / 2;
	const GLfloat leftTheta = M_PI / 2 - rightTheta;
	const GLfloat left = 2 * r * sinf(leftTheta);	//length of side of left little square
	const GLfloat right = 2 * r * sinf(rightTheta);	//length of side of right little square
	const GLfloat sum = left + right;
	
	GLfloat v[n * d] = {
		//three of the vertices of the right little square (going counterclockwise):
		r, 0.0f, -0.0f,
		r + right * cosf(rightTheta), right * sin(rightTheta), 0.0f,
		-r + sum * cosf(rightTheta), sum * sinf(rightTheta), 0.0f,
		
		//three of the vertices of the left little square (going clockwise):
		-r, 0.0f, 0.0f,
		-r - left * cosf(leftTheta), left * sinf(leftTheta), 0.0f,
		r - sum * cosf(leftTheta), sum * sinf(leftTheta), 0.0f
	};
	
	GLBatch b;
	b.Begin(GL_LINE_LOOP, n);
	b.CopyVertexData3f(v);
	b.End();
	b.Draw();
}
 

